%%% BEGIN erlaunch_eg.erl %%%
%%%
%%% erlaunch - execute selected Erlang functions from system command line
%%% Copyright (c)2003-2004 Cat's Eye Technologies.  All rights reserved.
%%%
%%% Redistribution and use in source and binary forms, with or without
%%% modification, are permitted provided that the following conditions
%%% are met:
%%%
%%%   Redistributions of source code must retain the above copyright
%%%   notice, this list of conditions and the following disclaimer.
%%%
%%%   Redistributions in binary form must reproduce the above copyright
%%%   notice, this list of conditions and the following disclaimer in
%%%   the documentation and/or other materials provided with the
%%%   distribution.
%%%
%%%   Neither the name of Cat's Eye Technologies nor the names of its
%%%   contributors may be used to endorse or promote products derived
%%%   from this software without specific prior written permission.
%%%
%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
%%% CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
%%% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
%%% DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
%%% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
%%% OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
%%% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
%%% OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
%%% ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
%%% OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
%%% OUT OF THE USE OF THIS SOFTWARE, EVEN IF AsDVISED OF THE
%%% POSSIBILITY OF SUCH DAMAGE. 

%% @doc Example commands for <code>erlaunch</code>.
%%
%% @end

-module(erlaunch_eg).
-vsn('2004.0309').
-author('cpressey@catseye.mine.nu').
-copyright('Copyright (c)2003-2004 Cat`s Eye Technologies. All rights reserved.').

-export([ls/0, raw_ls/4, io_test/0, exit_test/1, error_test/0]).

%% @spec ls() -> ok
%% @doc List the contents of the current directory (<code>mapped</code>).
%% This function can be called by the Erlang shell, or from the system
%% command line like <code>erl -run erlaunch_eg ls -run init stop</code>.
%% When called from <code>erlaunch</code>, it is <code>mapped</code>
%% appropriately so that the current directory is set to the caller's,
%% the I/O is routed to and from the caller, etc.

ls() ->
  {ok, Dir} = file:list_dir("."),
  lists:foreach(fun(File) ->
    io:fwrite("~p~n", [File])
  end, Dir),
  ok.

%% @spec raw_ls(erlaunch_context(), Args::[string()], Cwd::string(), Env::dict()) ->
%%   {exit, integer()}
%% @doc List the contents of the current directory (<code>raw</code>).
%% This function demonstrates how to use <code>erlaunch</code>'s
%% <code>raw</code> interface, and as such, cannot be used directly
%% from the Erlang shell or the system command line.

raw_ls(ErlaunchContext, _Args, Cwd, _Env) ->
  {ok, Dir} = file:list_dir(Cwd),
  lists:foreach(fun(File) ->
    erlaunch:fwrite(ErlaunchContext, "~p~n", [File])
  end, Dir),
  {exit, 0}.

%% @spec io_test() -> ok
%% @doc Test <code>mapped</code> I/O.

io_test() ->
  io:put_chars("io:put_chars"),
  io:nl(),
  io:write({io,write}),
  io:fwrite("io:~p~n", [fwrite]),

  % Chars = io:get_chars('io:get_chars> ', 10),
  % io:fwrite("~p~n", [Chars]),
  % Term = io:read('io:read> '),
  % io:fwrite("~p~n", [Term]),
  % Float = io:fread('io:fread> ', "~f"),
  % io:fwrite("~p~n", [Float]),

  io_test_loop("").

io_test_loop(eof) ->
  ok;
io_test_loop(_) ->
  Line = io:get_line('io:get_line> '),
  io:fwrite("~p~n", [Line]),
  io_test_loop(Line).

%% @spec exit_test(string()) -> {ok, string()} | {error, string()}
%% @doc Test how <code>erlaunch</code> translates return values to
%% exit codes.

exit_test(String) ->
  io:fwrite("~s~n", [String]),
  case hd(String) of
    $h ->
      {ok, String};
    _ ->
      {error, tl(String)}
  end.

%% @spec error_test() -> ok
%% @doc Test how <code>erlaunch</code> handles errors in commands.

error_test() ->
  1 = 2,
  ok.

%%% END of erlaunch_eg.erl %%%
